function genmin(action)

% Minimumsuche mittels genetischer Algorithmen
% 
% Author:  Klaus Lichtenegger
% Version: 1.0
% Date:    Dezember 20, 2001
% Status:  works quite well
% 
% User Interface basiert auf Game of Life von W. Kernbichler

% ==========================================================================
func_name = 'genmin';                      % function name - same as declaration
% ==========================================================================
handle_name = [func_name,'_HANDLE'];        % name of the global variable
eval(['global ',handle_name]);              % set it global
if nargin == 0,  action = 'initgraph'; end  % set to initgraph without input
if ~strcmp(action,'initgraph'), 
  eval(['HANDLE=',handle_name,';']);        % copy global to local
else

  % general outfit:
  logo_switch = 1;                  % logo: yes (1), no (0)
  fig_title   = 'Minfind';
  logo_title  = 'Minimumsuche mittels genetischer Algorithmen';
  logo_name   = 'Klaus Lichtenegger, Winfried Kernbichler';
  logo_lv     = 'Multimediale Lehre';
  logo_type   = 'logo_1';           % only this is defined at the moment
  
  % some special values:
  min_dim     = 5;                  % minimum dimension (x- and y-dir)
  max_dim     = 150;                % maximum dimension (x- and y-dir)
  sta_dim     = 40;                 % standard dimension (x- and y-dir)
  min_iter    = 1;                  % minimum number of iterations
  max_iter    = 200;                % maximum number of iterations
  sta_iter    = 100;                % standard number of iterations
  
  % some colors
  ax_color = [0.9,0.9,0.9];         % default axes color
  bo_color = [0.6,0.6,0.6];         % default border color
  
  % do not change
  HANDLE = [];                      % empty at the beginning

end

% ==========================================================================
% ==========================================================================
% Here every aspect of the graphics part should be handled
% This contains the definition and the callback routines
switch lower(action)
  
  % ========================================================================
  % initialization of program
  case 'initgraph'    
    % ----------------------------------------------------------------------
    % preprocessing for initgraph (keep this lines) 
    eval( [func_name,' ''initgraph_pre'''] );
    % ----------------------------------------------------------------------
    
    % ---------------------------------------------------------------------- 
    % definition and creation of graphics objects
    % ----------------------------------------------------------------------

    % ----------------------------------------------------------------------  
    % definition of main figure
    % ----------------------------------------------------------------------  
    F.figure = struct( ...
	'frame',    { [0.0, 0.0, 1.0, 1.0] }, ...
	'geometry', { [1] }, ...
	'closerequestfcn', { 'close_request_main' }, ...
	'renderer', { 'opengl' }, ...
	'name',     { fig_title }, ...
	'tag',      { 'main' } ...
	);
    % --- should be kept ---------------------------------------------------  
    [HANDLE,fig_han] = gui_create(gui_define(F),HANDLE);  clear F;
    % add function name to tag and closerequestfcn
    % important for finding program related figures and to find function
    gui_add_name(fig_han,func_name,'_','b','tag');
    gui_add_name(fig_han,func_name,' ','b','closerequestfcn');
    % sets necessary userdata for figures
    gui_set_userdata(fig_han, 'function',func_name, 'end','exit');
    figmain_han = fig_han;
    % figure color
    fig_color = get(gui_handle_query(HANDLE.figure,'tag','*main*'),'color');
    % ----------------------------------------------------------------------
    
    % ----------------------------------------------------------------------  
    % create meta frames
    % ----------------------------------------------------------------------  
    if logo_switch
      logo_y_pos  = 0.9; % y division between logo and rest
    else
      logo_y_pos  = 1.0;
    end % if logo_switch
    meta_frame_logo   = [0, logo_y_pos, 1, 1-logo_y_pos]; % logo
    meta_frame_layer1 = [0, 0,          1, logo_y_pos  ]; % layer 1   
    % ----------------------------------------------------------------------  

    % ----------------------------------------------------------------------  
    % create logo
    % ----------------------------------------------------------------------
    if logo_switch
      % --------------------------------------------------------------------  
      logo_string = {logo_title,logo_name,logo_lv};
      logo_parent = figmain_han;
      % here one could use HANDLE instead of HANDLE_LOGO, then one could 
      % refere to all contents within the rest of the program.
      % When using HANDLE_LOGO this graphics part stays seperate. 
      HANDLE_LOGO = [];
      HANDLE_LOGO = gui_create_logo(HANDLE_LOGO, logo_string, ...
	  logo_parent, meta_frame_logo, logo_type);
    end % if logo_switch
    % ----------------------------------------------------------------------  
    
    % ---------------------------------------------------------------------- 
    % create main frames at 5 positions
    %  four joined parts with division lines given at 
    %  x: x_div_pos y: y_div_pos
    % ----------------------------------------------------------------------  
    mf = meta_frame_layer1;    % choose meta frame
    x_div_pos = 0.70 * mf(3);  % division line in x direction
    y_div_pos = 0.20 * mf(4);  % division line in y direction
    y_div_ps2 = 0.85 * mf(4);  % 2nd y-division line (right hand side only)
   
    % frames for lower-left, upper-left, lower-right, mid-right, upper-right
    ll_frame = [ mf(1),     mf(2),     x_div_pos,       y_div_pos       ];
    ul_frame = [ mf(1),     y_div_pos, x_div_pos,       mf(4)-y_div_pos ];
    lr_frame = [ x_div_pos, mf(2),     mf(3)-x_div_pos, y_div_pos       ];
    mr_frame = [ x_div_pos, y_div_pos, mf(3)-x_div_pos, y_div_ps2-y_div_pos];
    ur_frame = [ x_div_pos, y_div_ps2, mf(3)-x_div_pos, mf(4)-y_div_ps2 ];
    
    % ----------------------------------------------------------------------  
    % definition of some basis axes system for the background of all five
    % parts defined in the meta frame.
    Fbasis.axes = struct( ...
	'geometry', { [1] }, ...
	'xborder',  { [0.0, 0.0] }, ...
	'yborder',  { [0.0, 0.0] }, ...
	'xgap',     { [0.0] }, ...
	'ygap',     { [0.0] }, ...	
	'parent',   { [1] }, ...
	'visible',  { 'on' }, ...
	'box',      { 'on' }, ...
	'xtick',    { [] }, ...
	'ytick',    { [] }, ...
	'color',    { ax_color }, ...
	'xcolor',   { bo_color }, ...
	'ycolor',   { bo_color } );
    
    
    % ======================================================================  
    % lower left position : sliders
    % ---------------------------------------------------------------------- 
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ll_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_left',' ','b','tag');            
    
    % ----------------------------------------------------------------------
    % definitions of slider values
    geom = [ 1, 1, 1, 1; 1, 1, 1, 1];
    % set important values
    visible = { 'on', 'on', 'on', 'on'; 'on', 'on',  'on', 'on'};
    minval  = { -5, -5, -10, -10; -5, -5, -5,  0 }; % minimum
    maxval  = {  5,  5,  10,   0;  5,  5,  5, 10 }; % maximum
    svalue  = { -2,  2,   3,   0; -2,  2, .1,  5 }; % value for slider
    evalue = svalue;                                % value for edit (string)
    for k = 1:length(svalue(:)), evalue{k} = num2str(svalue{k}); end

    % set strings = variable names
    cab = 'set_value';
    str = { 'x_min', 'x_max', 'x_start', 'z_min'; ...
            'y_min', 'y_max', 'y_start', 'z_max' };
    tag = lower(str);
    
    % create variables from string
    var = transpose(flipud(lower(str)));
    
    % callback action to be performed
    cb_no = ''; cbaction = { cb_no, cb_no, cb_no, cb_no; cb_no, cb_no, cb_no, cb_no };
    cbaction = transpose(flipud(lower(cbaction)));
    % what should be changed in the slider
    cb_ch = { 'value','round(##)' }; cb_no = {};
    cbchange_sl = { cb_no, cb_no, cb_no, cb_no; cb_no, cb_no, cb_no, cb_no}; 
    cbchange_sl = transpose(flipud(cbchange_sl));
    % what should be changed in the edit field
    cb_ch = { 'string','num2str((round(str2num(##))))' };	cb_no = {};
    cbchange_ed = { cb_no, cb_no, cb_no, cb_no; cb_no, cb_no, cb_no, cb_no };
    cbchange_ed = transpose(flipud(cbchange_ed));
    % what should be updated when slider changes, see also updatehandle
    update_sl = { { 'value','string','num2str(##)' } };
    update_sl = transpose(flipud(update_sl));
    % what should be updated when edit changes, see also updatehandle    
    update_ed = { { 'string','value','str2num(##)' } };
    update_ed = transpose(flipud(update_ed));

    Fu.uicontrol = struct( ...
	'frame',         { ll_frame }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.3] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { 'slider' }, ...
	'min',           { minval }, ...
	'max',           { maxval }, ...
	'value',         { svalue }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 1 }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);
    [HANDLE,slider_han] = gui_create(gui_define(Fu),HANDLE);
    
    % sets variable name in userdata + action and update information
    gui_set_userdata(slider_han,'variable',var); 
    gui_set_userdata(slider_han,'cbaction',cbaction);
    gui_set_userdata(slider_han,'cbchange',cbchange_sl);
    gui_set_userdata(slider_han,'updateproperty',update_sl); 
    gui_add_name(slider_han,'slider',' ','e','tag');    
    
    % add edit field shifted above (look for 'yshift')
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'left');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.065);
    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',              0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'edit');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'left');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              evalue);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.60);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            'edit_check');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,edit_han] = gui_create(gui_define(Fu),HANDLE);
    
    % sets variable name in userdata + action and update information
    gui_set_userdata(edit_han,'variable',var); 
    gui_set_userdata(edit_han,'cbaction',cbaction);
    gui_set_userdata(edit_han,'cbchange',cbchange_ed);
    gui_set_userdata(edit_han,'updateproperty',update_ed);
    gui_set_userdata(edit_han,'callback',[func_name,' ',cab]);
    gui_add_name(edit_han,'slider_edit',' ','e','tag');    

    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'right');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.12);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'right');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.8);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_han,'slider_text',' ','e','tag');    

    % updatehandle for slider (handles of edit)
    edit_cell = mat2cell(edit_han,1,ones(size(edit_han)));
    gui_set_userdata(slider_han,'updatehandle',edit_cell); 
    % updatehandle for edit (handles of slider)
    slider_cell = mat2cell(slider_han,1,ones(size(slider_han)));
    gui_set_userdata(edit_han,'updatehandle',slider_cell); 
    % adds function name to callback    
    gui_add_name([slider_han,edit_han],func_name,' ','b','callback');
    % adds common name to tag
    gui_add_name([slider_han,edit_han,text_han],'lower_left',' ','b','tag');
    % clears structur Fu    
    clear Fu
    % ======================================================================      
    
    
    % ======================================================================  
    % lower right position : Start, Help, Stop, Exit
    % ----------------------------------------------------------------------  
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',lr_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_right',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % define uicontrol pushbutton
    geom = [1,1 ; 1,1];
    str = reshape({ 'Start','Help', 'Stop', 'Exit'}, size(geom));
    tag = lower(str);
    cab = lower(str);
    
    F.uicontrol = struct( ...
	'frame',    { lr_frame }, ...
	'geometry', { geom }, ...
	'parent',   { [1] }, ...
	'style',    { 'pushbutton' }, ...
	'visible',  { 'on' }, ...
	'fontsize', { 0.3 }, ...
	'string',   { str }, ...
	'callback', { cab }, ...
	'tag',      { tag } ...
	);
    [HANDLE,newh] = gui_create(gui_define(F),HANDLE); clear F;
    gui_add_name(newh,func_name,' ','b','callback');
    gui_add_name(newh,'lower_right',' ','b','tag');            
    % settings for stop handle
    stop_handle = gui_handle_query(newh,'tag','*stop');
    gui_set_userdata(stop_handle,'stop',0);
    set(stop_handle,'enable','off');
    % ======================================================================  
    
    
    % ======================================================================  
    % upper left position : Main Image Matrix
    % ----------------------------------------------------------------------
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ul_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'upper_left',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % add axes for results
    F.axes = struct( ...
	'frame',    { ul_frame }, ...
	'geometry', { [1] }, ...
	'parent',   { [1] }, ...
	'visible',  { 'on' }, ...
	'box',      { 'off' }, ...
	'tag',      { 'imagemat' } ...
	);
    [HANDLE,axes_h] = gui_create(gui_define(F),HANDLE);clear F;

    axes_han   = HANDLE.axes;
    im_ax_han  = gui_handle_query(axes_han,'tag','*imagemat');
    figure_han = get(im_ax_han,'parent');    
    axes(im_ax_han); view(3);
        
    surf1_han = surface([],[],[],[]);
    set(surf1_han,'tag','imagemat');
    HANDLE = gui_create(surf1_han,HANDLE);             
    % ======================================================================  

    
    % ======================================================================  
    % upper right position : Return Values and Function
    % ----------------------------------------------------------------------  
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',ur_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'upper_right',' ','b','tag');             
        
    % text and edit
    geom = [1, 1; 1, 1; 2, 2];
    str = reshape( { 'x_best','y_best','testfunc', ...
          'x_b_val', 'y_b_val', ''}, size(geom));
    sty = reshape( { 'text','text','edit', 'text', 'text', 'edit'}, size(geom));
    tag = lower(str);
    str(3,1) = {'(2*exp(.1*x)-1.5*x.^2+.5*x.^4).*cosh(y)'};
            
    Ftext.uicontrol = struct( ...
	'frame',    { ur_frame }, ...
	'geometry', { geom }, ...
	'yborder',  { [0.05,0.05] }, ...
	'ygap',     { [0.02] }, ...
	'parent',   { [1] }, ...
	'style',    { sty }, ...
	'visible',  { 'on' }, ...
	'fontsize', { 0.6 }, ...
	'string',   { str }, ...
	'tag',      { tag }, ...
	'value',    { 1 } ...
	);
    
    [HANDLE,newh] = gui_create(gui_define(Ftext),HANDLE); clear Ftext;
    % ======================================================================  
  
    
    % ======================================================================  
    % middle right position: additional sliders (algorithm parameters)
    % ----------------------------------------------------------------------  
    % basic axis for background    
    Fbasis.axes = setfield(Fbasis.axes,'frame',mr_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'mid_right',' ','b','tag');             
    
    % ----------------------------------------------------------------------
    % definitions of slider values
    geom = ones(8,1);
    % set important values
    visible = { 'on'; 'on'; 'on'; 'on'; 'on'; 'on'; 'on';  'on' };
    minval  = {    6;    1;    0;    0;    0;    0;    0;     0 }; % minimum
    maxval  = { 1000;  100;   10;    1;    1;    1;   .1;     1 }; % maximum
    svalue  = {   60;   20;    1;   .2;   .1;   .5;  .01;     1 }; % value for slider
    evalue = svalue;                                               % value for edit (string)
    for k = 1:length(svalue(:)), evalue{k} = num2str(svalue{k}); end

    % set strings = variable names
    cab = 'set_value';
    str = { 'Population'; 'Generationen'; 'Mut1'; 'P_Mut1'; ...
            'Mut2'; 'P_Mut2'; 'Mut3'; 'P_mut3' };
    tag = lower(str);
    
    % create variables from string
    var = transpose(flipud(lower(str)));
    
    % callback action to be performed
    cb_no = ''; cbaction = { cb_no; cb_no; cb_no; cb_no; cb_no; cb_no; cb_no; cb_no };
    cbaction = transpose(flipud(lower(cbaction)));
    % what should be changed in the slider
    cb_ch = { 'value','round(##)' }; cb_no = {};
    cbchange_sl = { cb_no; cb_no; cb_no; cb_no; cb_no; cb_no; cb_no; cb_no}; 
    cbchange_sl = transpose(flipud(cbchange_sl));
    % what should be changed in the edit field
    cb_ch = { 'string','num2str((round(str2num(##))))' };	cb_no = {};
    cbchange_ed = { cb_no; cb_no; cb_no; cb_no; cb_no; cb_no; cb_no; cb_no };
    cbchange_ed = transpose(flipud(cbchange_ed));
    % what should be updated when slider changes, see also updatehandle
    update_sl = { { 'value','string','num2str(##)' } };
    update_sl = transpose(flipud(update_sl));
    % what should be updated when edit changes, see also updatehandle    
    update_ed = { { 'string','value','str2num(##)' } };
    update_ed = transpose(flipud(update_ed));

    Fu.uicontrol = struct( ...
	'frame',         { mr_frame }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.1, 0.1] }, ...
	'yborder',       { [0.1, 0.1] }, ...
	'xgap',          { [0.1] }, ...
	'ygap',          { [0.07] }, ...
	'xshift',        { [0.0] }, ...
	'yshift',        { [0.0] }, ...
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { 'slider' }, ...
	'min',           { minval }, ...
	'max',           { maxval }, ...
	'value',         { svalue }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 1 }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);
    [HANDLE,slider_han] = gui_create(gui_define(Fu),HANDLE);
    
    % sets variable name in userdata + action and update information
    gui_set_userdata(slider_han,'variable',var); 
    gui_set_userdata(slider_han,'cbaction',cbaction);
    gui_set_userdata(slider_han,'cbchange',cbchange_sl);
    gui_set_userdata(slider_han,'updateproperty',update_sl); 
    gui_add_name(slider_han,'slider',' ','e','tag');    
    
    % add edit field shifted above (look for 'yshift')
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'left');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.35);
    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',              0.05);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'edit');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'left');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              evalue);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.60);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            'edit_check');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,edit_han] = gui_create(gui_define(Fu),HANDLE);
    
    % sets variable name in userdata + action and update information
    gui_set_userdata(edit_han,'variable',var); 
    gui_set_userdata(edit_han,'cbaction',cbaction);
    gui_set_userdata(edit_han,'cbchange',cbchange_ed);
    gui_set_userdata(edit_han,'updateproperty',update_ed);
    gui_set_userdata(edit_han,'callback',[func_name,' ',cab]);
    gui_add_name(edit_han,'slider_edit',' ','e','tag');    

    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'right');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.45);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'right');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.8);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_han,'slider_text',' ','e','tag');    

    % updatehandle for slider (handles of edit)
    edit_cell = mat2cell(edit_han,1,ones(size(edit_han)));
    gui_set_userdata(slider_han,'updatehandle',edit_cell); 
    % updatehandle for edit (handles of slider)
    slider_cell = mat2cell(slider_han,1,ones(size(slider_han)));
    gui_set_userdata(edit_han,'updatehandle',slider_cell); 
    % adds function name to callback    
    gui_add_name([slider_han,edit_han],func_name,' ','b','callback');
    % adds common name to tag
    gui_add_name([slider_han,edit_han,text_han],'lower_left',' ','b','tag');
    % clears structur Fu    
    clear Fu
    % ======================================================================      
    
    
    % ----------------------------------------------------------------------  
    % update handle (keep these lines)
    % ----------------------------------------------------------------------  
    eval( [handle_name,'=HANDLE;'] );
    
    % ----------------------------------------------------------------------  
    % postprocessing for initgraph (keep these lines)
    % ----------------------------------------------------------------------  
    eval( [func_name,' ''initgraph_post'''] );
    % ----------------------------------------------------------------------  
    
    % ----------------------------------------------------------------------  
    % set main figure to visible on (keep these lines)   
    % ----------------------------------------------------------------------  
    fig_han = eval([handle_name,'.figure']);   % get content of figure
    fig_han = gui_handle_query(fig_han,'tag',['*main']);
    set(fig_han,'visible','on');          % set figures to visible  
    
    % ======================================================================
   
  case 'initgraph_pre'               
    % preprocessing for initgraph (keep these lines)    
    % ======================================================================
    gui_default('set') % default values
    % close figures belonging to this program
    fig_han = get(0,'children');   
    fig_han = gui_handle_query(fig_han, ...
	'type','figure','tag',[func_name,'*']);
    delete(fig_han);
    % ======================================================================
  
  case 'initgraph_post'               
    % postprocessing for initgraph (keep these lines)   
    % ======================================================================
    gui_default('remove'); % reset defaults to MATLAB defaults
    % ======================================================================
  
  case 'exit'
  % finishing the program (keep these lines)
  % ========================================================================
    fig_han = eval([handle_name,'.figure']); % get all figures
    fig_han = fig_han(ishandle(fig_han));    % check if exist as handle
    delete(fig_han);                         % closes all figures
    clear('global',handle_name)              % deletes the global variable
    % ======================================================================
  
  case 'close_request_main'  
    % close request main (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function'); % function name
    exit = gui_get_userdata(gcf,'end');      % action for exit
    msg = { ...
	['Closing this figure exits program ',func,'.']; ...
	'Do you really want to exit?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end
    % ======================================================================
    
  case 'close_request_help'  
    % close request for help (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function');
    exit = gui_get_userdata(gcf,'end');
    msg = { ...
	'Closing this figure is not possible.'; ...
	'Do you want to make it invisible?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end    
    % ======================================================================
        
  case 'help'
    % calling help (keep these lines)
    % ======================================================================
    frame = [0.1, 0.1, 0.8, 0.5];
    [HANDLE,new] = gui_help(HANDLE,frame,func_name);
    if new, eval([handle_name,'=HANDLE;']); end % export to global
    % ======================================================================
  
  case 'edit_check'
    % checks string from edit field (keep these lines)
    % ======================================================================
    str    = get(gcbo,'string');
    oldval = get(gcbo,'value');
    minval = get(gcbo,'min');
    maxval = get(gcbo,'max');
    val = str2num(str);
    % handle all error cases
    if ~isempty(val)
      if val<minval
	msg = { 'Value to small!','Value set to minimum.' };
	e_han = msgbox(msg,'Edit Box','error','modal');
	set(gcbo,'string',num2str(minval));
      elseif val>maxval
	msg = { 'Value to large!','Value set to maximum.' };
	e_han = msgbox(msg,'Edit Box','error','modal');
	set(gcbo,'string',num2str(maxval));
      end
    else
      msg = { 'Wrong value!','Value set to old value.' };
      e_han = msgbox(msg,'Edit Box','error','modal');
      if isfield(get(gcbo,'userdata'),'updatehandle')
	updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
	updateproperty = gui_get_userdata(gcbo,'updateproperty');
	oldval = get(updatehandle,updateproperty{2});
      end
      set(gcbo,'string',num2str(oldval));
    end
    
    % do the real callback
    if isfield(get(gcbo,'userdata'),'callback')
      eval( gui_get_userdata(gcbo,'callback') );
    end
    % ======================================================================
    
    
  case 'set_value'
    % get values from caller 
    % ======================================================================
    e_str  = gui_get_userdata(gcbo,'variable');
    cbaction = gui_get_userdata(gcbo,'cbaction');
    figure_han = get(gcbo,'parent');
    func_name = gui_get_userdata(figure_han,'function');
    
    if isfield(get(gcbo,'userdata'),'cbchange')
      cbchange = gui_get_userdata(gcbo,'cbchange');
      if ~isempty(cbchange)
	prop = cbchange{1};
	chfunc = cbchange{2};
	e_str = [ 'get(gcbo,''',prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(gcbo,''',prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
    
    % do the update of another handle
    if isfield(get(gcbo,'userdata'),'updatehandle')
      updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
      updateproperty = gui_get_userdata(gcbo,'updateproperty');      
      if ~isempty(updatehandle)
	from_prop = updateproperty{1};
	to_prop   = updateproperty{2};
	chfunc    = updateproperty{3};
	e_str = [ 'get(gcbo,''',from_prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(updatehandle,''',to_prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
    
    if ~isempty(cbaction)
      eval( [func_name,' ',cbaction] );
    end
  
    
  % ======================================================================    
  % Start: start simulation
  % ======================================================================
 
  case 'start'
   
    % get uicontrol handle
    uicontrol_han = HANDLE.uicontrol;

    % get handles of function edit and x_best-, y_best-values
    testfun_han = gui_handle_query(uicontrol_han,'tag','*testfunc*');
    xbest_han = gui_handle_query(uicontrol_han,'tag','*x_b_val*');
    ybest_han = gui_handle_query(uicontrol_han,'tag','*y_b_val*');    
    testfunc = get(testfun_han, 'String');       

    % test if function is valid
    % special thanks to Dieter Mayer for the "try-catch"-hint
    x=1:10; y=1:10;
    try,
        eval(['z=',testfunc,';']);
    catch,
        msg = { 'Function not valid' };
	    e_han = msgbox(msg,'Edit Box','error','modal');
        return;
    end
    
    % get handles of function edit and x_best-, y_best-values
    xbest_han = gui_handle_query(uicontrol_han,'tag','*x_b_val*');
    ybest_han = gui_handle_query(uicontrol_han,'tag','*y_b_val*');
    stop_han = gui_handle_query(HANDLE.uicontrol,'tag','*stop');    

    % set STOP to Zero
    gui_set_userdata(stop_han,'stop',0);
    
    % get slider handles
    uicontrol_han = HANDLE.uicontrol;
    data_handle = gui_handle_query(uicontrol_han,'tag','*slider');
    
    % get variables from slider values
    variables = gui_get_userdata(data_handle,'variable');
    values    = get(data_handle,'value');
    lv = length(values);
    if lv==1
      eval([variables,'=values;'])
    else
      for k = 1:lv
	value = values{k};
	var   = variables{k};
	eval([var,'=value;'])
      end
    end
        
    % get axes handles:
    axes_han = HANDLE.axes;
    axes_han = gui_handle_query(axes_han,'tag','*imagemat*');

    % disable buttons and sliders:
    set(uicontrol_han,'enable','off');
    set(xbest_han,'enable','on');
    set(ybest_han,'enable','on');
    set(stop_han,'enable','on');
    
    % Werte von sliders holen:
    start =    [x_start, y_start];        % Startwerte fr Suche (nahe lokalem Minimum)
    mut =      [mut1, mut2, mut3];        % Mutagenitten
    p_mut =    [p_mut1, p_mut2, p_mut3];  % Mutationswahrscheinlichkeiten
    number =   floor(population);         % Zahl der Individuen/Schritt
    fititer =  floor(generationen);       % Zahl der Schritte
    fithand =  axes_han;
    
    % Hilfsgren definieren:
    
    step1 = mut(1); schranke1 = p_mut(1);
    step2 = mut(2); schranke2 = p_mut(2);
    step3 = mut(3); schranke3 = p_mut(3);
    n1 = floor(number/2);
    n2 = floor(5*number/6);

    % Zufllige Ausgangsdaten generieren
    % (um Startwerte normalverteilt):
    DNA_Data = randn(number, 2);
    DNA_Data(:,1) = DNA_Data(:,1) + start(1);
    DNA_Data(:,2) = DNA_Data(:,2) + start(2);

    
    % Schleife fr Durchlufe
    
    for I = 1:fititer
   
      % Neue "Hitliste" generieren:
      %           [ x_best  y_best  funcwert1 ]
      % hitlist = [ x_2bst  y_2bst  funcwert2 ]
      %           [ x_3bst  y_3bst  funcwert3 ]
      hitlist = zeros(3,3); hitlist(:,3) = inf;
   
      % ====================================================
      % Die Funktionswerte der (x,y)-Paare werden nun 
      % miteinander verglichen, es wird eine Art "Ranking" 
      % erstellt, also die besten Werte ermittelt.
      % ====================================================
   
      % Alle DNAs testen:
      for J = 1:number
            
         % Koordinaten auslesen:
         x = DNA_Data(J,1); y = DNA_Data(J,2);
      
         % Funktionswert ermitteln
         eval(['funcwert = ' testfunc ';']);
      
         % Wenn in den Top 3
         if funcwert <= hitlist(3,3)
         
            % Wenn in den Top 2
            if funcwert <= hitlist(2,3)
            
               % Wenn bislang bester Wert
               if funcwert <= hitlist(1,3)
               
                  % Auf Nr.1 setzen, andere nach unten
                  hitlist(3,:)=hitlist(2,:);
                  hitlist(2,:)=hitlist(1,:);
                  hitlist(1,:)=[x, y, funcwert];
               
               else % Platz Zwei
               
                  % Auf Nr.2 setzen, Zwei auf Drei
                  hitlist(3,:)=hitlist(2,:);
                  hitlist(2,:)=[x, y, funcwert];
               
               end %if (bester Wert)
            
            else % Platz Drei
            
               % Auf Nr.3 setzen
               hitlist(3,:)=[x, y, funcwert];
            
            end %if in Top 2
         
         end %if in Top 3
      
      end %for J
      % alle DNAs getestet
   
      % ==================================================
      % == Jetzt werden die besten Werte repliziert,    ==
      % == dabei knnen und sollen Mutationen auftreten ==
      % ==================================================
   
      % besten DNAs sichern
      for J=1:3
         DNA_Data(J,1) = hitlist(J,1);
         DNA_Data(J,2) = hitlist(J,2);
      end % for
      
      % restliche Pltze mit Kopien besetzen
      for J=4:n1
         DNA_Data(J,1) = hitlist(1,1);
         DNA_Data(J,2) = hitlist(1,2);
      end % for J
      for J=n1+1:n2
         DNA_Data(J,1) = hitlist(2,1);
         DNA_Data(J,2) = hitlist(2,2);
      end % for J
      for J=n2+1:number
         DNA_Data(J,1) = hitlist(3,1);
         DNA_Data(J,2) = hitlist(3,2);
      end % for J
   
      % Mit p=schranke1 Mutation um O(step1)
      yesno = rand(number-3,1) < schranke1;
      DNA_Data(4:number,1:2) = DNA_Data(4:number,1:2) + step1*[yesno, yesno].*randn(number-3,2);
      % Mit p=schranke2 Mutation um O(step2)
      yesno = rand(number-3,1) < schranke2;
      DNA_Data(4:number,1:2) = DNA_Data(4:number,1:2) + step2*[yesno, yesno].*randn(number-3,2);
      % Mit p=schranke3 Mutation um O(step3)
      yesno = rand(number-3,1) < schranke3;
      DNA_Data(4:number,1:2) = DNA_Data(4:number,1:2) + step3*[yesno, yesno].*randn(number-3,2);
   
   
      % =============================================
      % Ergebnisse werden jetzt graphisch dargestellt
      % =============================================
   
      surf_han = HANDLE.surface;
      surf_han = gui_handle_query(surf_han,'tag','*imagemat*');
      
      if (x_min>x_max)|(y_min>y_max)|(z_min>z_max)
    	disp ('x_min>x_max oder y_min>y_max oder z_min>z_max');
    	break
      end % if
      
      % get view area
      x_vec = x_min:(x_max-x_min)/99:x_max;
      y_vec = y_min:(y_max-y_min)/99:y_max;
      [x,y] = meshgrid(x_vec, y_vec);
      eval( ['z = ', testfunc, ';'] );
    
      % display function
      axes(axes_han); shading interp;
      set(axes_han, 'ZLim', [z_min, z_max]);
      set(surf_han, 'XData', x_vec, 'YData', y_vec, 'ZData', z, 'CData', z);
      shading interp; hold on;
      rotate3d on;
      
      % display data points
      x = DNA_Data(:,1); y = DNA_Data(:,2);
      eval( ['z = ', testfunc, '+0.001;'] ); pkt_a = plot3(x,y,z,'r*');
      eval( ['z = ', testfunc, '-0.001;'] ); pkt_b = plot3(x,y,z,'r*');
      xlim([x_min, x_max]); ylim([y_min, y_max]);
      hold off; drawnow;
      delete(pkt_a); delete(pkt_b);
      
      % display best values
      set(xbest_han, 'String', num2str(hitlist(1,1)));
      set(ybest_han, 'String', num2str(hitlist(1,2)));
        
      % Abfrage ob STOP-Knopf gedrckt
      gestoppt = gui_get_userdata(stop_han,'stop');
      if gestoppt
	    break;
      end % if
      
   end %for I
       
   % enable buttons and sliders, disable stop button   
   set(uicontrol_han,'enable','on');
   set(stop_han,'enable','off');
   
   % allow 3D-ROTATE
   rotate3d on;
   
    
  % ======================================================================    
  % Stop: stop calculation
  % ======================================================================
    
  case 'stop'
    stop_han = gui_handle_query(HANDLE.uicontrol,'tag','*stop');    
    gui_set_userdata(stop_han,'stop',1);
  
  % ========================================================================

  
  otherwise
    % ======================================================================
    % warnings if callback routine is not defined (keep this at the end)    
    % ======================================================================
    disp(['Callback not defined: ',action]);
    % ======================================================================

end










